@@ -101,6 +101,7 @@
 //!   - Dependencies are full of unsafe
 //!
 //! # Changelog
+//! - v0.3.3 - Eliminate spurious "resumed after completion" worker thread panics.
 //! - v0.3.2 - Re-export `safina_sync::Receiver` and `safina_threadpool::NewThreadPoolError`.
 //! - v0.3.1 - Use `safina-async` v0.2.1.
 //! - v0.3.0 - `schedule_blocking` to return new `safina_sync::Receiver`.
@@ -318,7 +319,8 @@ impl Executor {
     /// Or use [`pin_utils::pin_mut`](https://docs.rs/pin-utils/latest/pin_utils/macro.pin_mut.html)
     /// to do it with unsafe code that does not allocate memory.
     pub fn spawn_unpin(self: &Arc<Self>, fut: impl (Future<Output = ()>) + Send + Unpin + 'static) {
-        let task = Arc::new(Mutex::new(fut));
+        let task: Arc<Mutex<Option<Box<dyn Future<Output = ()> + Send + Unpin>>>> =
+            Arc::new(Mutex::new(Some(Box::new(fut))));
         let weak_self = Arc::downgrade(self);
         self.async_pool.schedule(move || poll_task(task, weak_self));
     }
@@ -414,25 +416,35 @@ where
     }
 }
 
-fn poll_task(task: Arc<Mutex<dyn Future<Output = ()> + Send + Unpin>>, executor: Weak<Executor>) {
+#[allow(clippy::needless_pass_by_value)]
+fn poll_task(
+    task: Arc<Mutex<Option<Box<dyn Future<Output = ()> + Send + Unpin>>>>,
+    executor: Weak<Executor>,
+) {
     if executor.strong_count() > 0 {
-        let task_clone = task.clone();
-        let waker = std::task::Waker::from(Arc::new(TaskWaker::new(task, executor.clone())));
+        let waker =
+            std::task::Waker::from(Arc::new(TaskWaker::new(task.clone(), executor.clone())));
         let mut cx = std::task::Context::from_waker(&waker);
-        let mut fut_guard = task_clone.lock().unwrap();
-        let _guard = set_thread_executor(executor);
-        let _ = Pin::new(&mut *fut_guard).poll(&mut cx);
+        let mut opt_fut_guard = task.lock().unwrap();
+        if let Some(fut) = opt_fut_guard.as_mut() {
+            let _guard = set_thread_executor(executor);
+            match Pin::new(&mut *fut).poll(&mut cx) {
+                Poll::Ready(()) => {
+                    opt_fut_guard.take();
+                }
+                Poll::Pending => {}
+            }
+        }
     }
 }
 
-// TODO(mleonhard) Make TaskWaker and its clones single-use.
 struct TaskWaker {
-    task: Arc<Mutex<dyn Future<Output = ()> + Send + Unpin>>,
+    task: Arc<Mutex<Option<Box<dyn Future<Output = ()> + Send + Unpin>>>>,
     executor: Weak<Executor>,
 }
 impl TaskWaker {
     pub fn new(
-        task: Arc<Mutex<dyn Future<Output = ()> + Send + Unpin>>,
+        task: Arc<Mutex<Option<Box<dyn Future<Output = ()> + Send + Unpin>>>>,
         executor: Weak<Executor>,
     ) -> Self {
         Self { task, executor }
@@ -495,7 +507,8 @@ pub fn spawn(fut: impl (Future<Output = ()>) + Send + 'static) {
 /// Panics if the caller is not running on an [`Executor`](struct.Executor.html).
 pub fn spawn_unpin(fut: impl (Future<Output = ()>) + Send + Unpin + 'static) {
     if let Some(executor) = get_thread_executor() {
-        let task = Arc::new(Mutex::new(fut));
+        let task: Arc<Mutex<Option<Box<dyn Future<Output = ()> + Send + Unpin>>>> =
+            Arc::new(Mutex::new(Some(Box::new(fut))));
         let executor_weak = Arc::downgrade(&executor);
         executor
             .async_pool
